// Collide.h: interface for the CCollide class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_COLLIDE_H__E3F57089_A057_4886_8BA2_DD8476C45089__INCLUDED_)
#define AFX_COLLIDE_H__E3F57089_A057_4886_8BA2_DD8476C45089__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <d3d8.h>			
#include <d3dx8math.h>
#include <d3dx8tex.h>
#include <d3d8types.h>
#include <dinput.h>
#include <D3dx8core.h>

#define PI 3.141592654f

typedef struct _VERTEX {
    float x, y, z;     // position
    float nx, ny, nz;  // normal
    DWORD Diffuse;     // diffuse color
    float tu1, tv1;    // texture coordinates
	} VERTEX, *LPVERTEX;

typedef struct _TRIANGLE {
	char flag;
	VERTEX v[3];
	} TRIANGLE, *LPTRIANGLE;

class CCollide {
	public:
		float EPSILON;
		float SECONDARYEPSILON;

		BOOL tag;	// True to color impacted surfaces

		LPTRIANGLE *triangles;
		int triangle_count;

		struct TCollisionPacket {
			// data about player movement
			D3DXVECTOR3 velocity;
			D3DXVECTOR3 sourcePoint;

			// radius of ellipsoid.  
			D3DXVECTOR3 eRadius;
			  
			// for error handling  
			D3DXVECTOR3 lastSafePosition;
			BOOL stuck; 
			    
			// data for collision response 
			BOOL foundCollision;
			float  nearestDistance; // nearest distance to hit
			D3DXVECTOR3 nearestIntersectionPoint; // on sphere
			D3DXVECTOR3 nearestPolygonIntersectionPoint; // on polygon

			} collision;


		D3DXVECTOR3 Move(D3DXVECTOR3 Position, D3DXVECTOR3 Velocity, float Radius);
		D3DXVECTOR3 CollideWithWorld(D3DXVECTOR3 position, D3DXVECTOR3 velocity, float Radius);
		void CheckCollision(TCollisionPacket* colPackage);
		void Tweak(float epsilon, float secondaryepsilon);

		CCollide();
		virtual ~CCollide();
	};

#endif // !defined(AFX_COLLIDE_H__E3F57089_A057_4886_8BA2_DD8476C45089__INCLUDED_)
